class Key {
  static get DATABASE_PREFIX() {
    return "key_";
  }

  static get REDIS_CLIENT() {
    let config = require("config");
    let Redis = require("ioredis");

    return new Redis({
      host: config.get("redis.host"),
    });
  }

  static get EXPIRATION_AFTER_CHARGE_DAYS() {
    return 365;
  }

  static get KEY_CHARSET() {
    return require("config").get("keys.charset");
  }

  static get KEY_LENGTH() {
    return require("config").get("keys.length");
  }

  /**
   *
   * @returns A new MetaGer-Pass Key with 0 searches
   */
  static async CREATE_NEW_KEY(expire_at) {
    if (!expire_at) {
      let Dayjs = require("dayjs");
      expire_at = new Dayjs();
      expire_at = expire_at.add(6, "hour").unix();
    }
    return new Promise(async (resolve, reject) => {
      let redis_client = Key.REDIS_CLIENT;

      do {
        let key = "";

        for (let i = 0; i < Key.KEY_LENGTH; i++) {
          key +=
            Key.KEY_CHARSET[Math.floor(Math.random() * Key.KEY_CHARSET.length)];
        }

        let key_created = await redis_client.setnx(
          Key.DATABASE_PREFIX + key,
          0
        );
        if (key_created === 1) {
          await redis_client.expireat(Key.DATABASE_PREFIX + key, expire_at);
          return resolve(key);
        }
        await new Promise((resolve) => setTimeout(resolve, 50));
      } while (true);
    });
  }

  static async CHARGE_EXISTING_KEY(key, amount) {
    return new Promise(async (resolve, reject) => {
      let redis_client = Key.REDIS_CLIENT;

      // Check if key exists and is eligable for recharge
      redis_client.get(Key.DATABASE_PREFIX + key).then(async key_charge => {
        if (!key_charge || key_charge >= 3000) {
          reject("Der angegebene Schlüssel ist ungültig, oder beinhaltet bereits mehr als 3.000 Suchanfragen");
        } else {
          let dayjs = require("dayjs");
          let expiration = dayjs().add(Key.EXPIRATION_AFTER_CHARGE_DAYS, "day");
          redis_client.pipeline().incrby(Key.DATABASE_PREFIX + key, amount).expireat(Key.DATABASE_PREFIX + key, expiration.unix()).exec().then(result => {
            resolve({
              status: "SUCCESS",
              metager_pass_key: {
                key: key,
                searches: result[0][1],
                expire_at: expiration.format()
              }
            });
          }).catch(reason => {
            reject(reason);
          });

        }
      });
    });
  }
}

module.exports = Key;
