const uuid_generator = require("uuid");
const BlindSignature = require("blind-signatures");

var metager_pass_order_id = null;
var metager_pass_expires_at = null;
var metager_pass_sales_receipts = [];
var metager_pass_encrypted_sales_receipts = []; // Stores the encrypted sales receipts that get signed by our server after successful purchase
var metager_pass_encrypted_sales_receipts_r = []; // Those are the secrets for decrypting the signed sales_receipts; Should never leave the clients computer
var metager_pass_signatures = [];

one_generate_encrypted_sales_receipt();

function one_generate_encrypted_sales_receipt() {
  let current_step_container = document.getElementById(
    "generate-sales-receipt"
  );
  let next_step_container = document.getElementById("execute-payment");

  let amount = parseInt(document.querySelector("input[name=amount]").value);
  let unit_size = parseInt(
    document.querySelector("input[name=unit_size]").value
  );
  let N = document.querySelector("input[name=public_key_n]").value;
  let E = document.querySelector("input[name=public_key_e]").value;

  let ticket_count = (amount * unit_size) / 50;
  if (ticket_count % 1 !== 0) {
    console.error(
      "This should not happen. Ticket count should only produce integer values"
    );
    return;
  }

  for (let i = 0; i < ticket_count; i++) {
    let uuid = uuid_generator.v4();
    metager_pass_sales_receipts.push(uuid);
    let { blinded, r } = BlindSignature.blind({
      message: uuid,
      N: N,
      E: E,
    });
    metager_pass_encrypted_sales_receipts.push(blinded.toString());
    metager_pass_encrypted_sales_receipts_r.push(r.toString());
  }

  current_step_container.classList.remove("current");
  current_step_container.classList.add("finished");
  next_step_container.classList.add("current");

  two_execute_payment();
}

function two_execute_payment() {
  let current_step_container = document.getElementById("execute-payment");
  let development_handler_button = document.getElementById(
    "payment_method_development"
  );
  if (development_handler_button) {
    let development_handler = require("./checkout_development");
    development_handler_button.addEventListener("click", () => {
      development_handler(metager_pass_encrypted_sales_receipts);
    });
  }

  let paypal_handler = require("./checkout_paypal"); // Add handler for PayPal Checkout
  document
    .getElementById("payment_method_paypal")
    .addEventListener("click", () => {
      paypal_handler(metager_pass_encrypted_sales_receipts);
    });

  current_step_container.addEventListener("payment-complete", (e) => {
    current_step_container.classList.remove("current");
    current_step_container.classList.add("finished");
    three_verify_and_decrypt(e.detail);
  });
}

function three_verify_and_decrypt(signatures) {
  let current_step_container = document.getElementById("verify-and-decrypt");
  let next_step_container = document.getElementById("finish-purchase");

  current_step_container.classList.add("current");
  metager_pass_order_id = signatures.order_id;
  let dayjs = require("dayjs");
  metager_pass_expires_at = dayjs(signatures.expires_at).format("YYYY-MM");

  let N = document.querySelector("input[name=public_key_n]").value;
  let E = document.querySelector("input[name=public_key_e]").value;
  // Decrypting/Unblinding Signatures
  for (let i = 0; i < signatures.signatures.length; i++) {
    metager_pass_signatures.push(
      BlindSignature.unblind({
        signed: signatures.signatures[i],
        N: N,
        r: metager_pass_encrypted_sales_receipts_r[i],
      }).toString()
    );
  }

  // Verify Signatures
  // ToDo do something with invalid signatures
  for (let i = 0; i < metager_pass_sales_receipts.length; i++) {
    let verification = BlindSignature.verify({
      unblinded: metager_pass_signatures[i],
      N: N,
      E: E,
      message: metager_pass_sales_receipts[i],
    });
    console.log(metager_pass_sales_receipts[i] + " => " + verification);
  }

  current_step_container.classList.remove("current");
  current_step_container.classList.add("finished");
  four_finish_purchase();
}

function four_finish_purchase() {
  let current_step_container = document.getElementById("finish-purchase");
  current_step_container.classList.add("current");

  // Prepare Voucher download
  let voucher_link = document.getElementById("voucher-link");
  let voucher_data = {
    expiration_month: metager_pass_expires_at,
    receipts: [],
  };
  for (let i = 0; i < metager_pass_sales_receipts.length; i++) {
    voucher_data.receipts.push({
      receipt: metager_pass_sales_receipts[i],
      signature: metager_pass_signatures[i],
    });
  }
  voucher_data =
    "data:application/mgpass;charset=utf-8;base64," +
    btoa(JSON.stringify(voucher_data, null, 4));
  voucher_link.href = voucher_data;
  voucher_link.download = "mgpass_coupon.json";

  // Create redeem data
  let order_month = require("dayjs")
    .unix(metager_pass_order_id.substr(0, 10))
    .format("YYYY-MM");
  let redeem_data = {
    expiration_month: metager_pass_expires_at,
    generation_month: order_month,
    metager_pass_codes: [],
  };
  for (let i = 0; i < metager_pass_sales_receipts.length; i++) {
    redeem_data.metager_pass_codes.push({
      code: metager_pass_sales_receipts[i],
      signature: metager_pass_signatures[i],
    });
  }

  // Make Create Key button work
  let create_key_button = document.getElementById("create-key-button");
  create_key_button.addEventListener("pointerdown", e => {
    if (e.target.disabled) {
      return;
    } else {
      e.target.disabled = true;
    }

    fetch("/redeem/create", {
      method: "POST",
      headers: {
        "Content-Type": "application/json;charset=utf-8",
      },
      body: JSON.stringify(redeem_data),
    }).then(response => {
      if (!response.ok) {
        return response.json().then(json_response => {
          console.error(json_response);
          for (let i = 0; i < json_response.errors.length; i++) {
            let error_message = document.createElement("li");
            error_message.textContent = json_response.errors[i].msg
            document.querySelector("#create-key .errors ul").appendChild(error_message);
          }
          e.target.disabled = false;
          return Promise.reject("Error while creating a new MetaGer-Pass Key");
        });
      } else {
        return response.json()
      }
    })
      .then(response => {
        let key = response.metager_pass_key;
        // Prefill the recharge existing key button
        let metager_pass_key_input = document.getElementById("metager-pass-key");
        metager_pass_key_input.value = key.key;
        document.getElementById("metager-pass-key").addEventListener("change", e => {
          // When the user changes the generated key unlock the generate button again
          create_key_button.disabled = false;
        });
        // Select The recharge existing key tab and show the correct text
        document.getElementById("recharge-existing-key").classList.add("hidden");
        document.getElementById("recharge-new-key").classList.remove("hidden");
        document.getElementById("recharge-key-radio").checked = true;
      });
  });

  // Make Redeem Button Work
  let redeem_button = document.getElementById("recharge-key-button");
  redeem_button.addEventListener("pointerdown", e => {
    if (e.target.disabled) {
      return;
    } else {
      e.target.disabled = true;
      document.getElementById("metager-pass-key").disabled = true;
    }
    let post_data = { ...redeem_data };
    post_data.metager_pass_key = document.getElementById("metager-pass-key").value;
    fetch("/redeem", {
      method: "POST",
      headers: {
        "Content-Type": "application/json;charset=utf-8",
      },
      body: JSON.stringify(post_data),
    }).then(response => response.json())
      .then(response => {
        // Show Summary and fill data for it
        let dayjs = require('dayjs');
        document.querySelector("#redeem-successful .key").textContent = response.metager_pass_key.key;
        document.querySelector("#redeem-successful .searches").textContent = response.metager_pass_key.searches;
        document.querySelector("#redeem-successful .valid-until").textContent = dayjs(response.metager_pass_key.expire_at).format("DD.MM.YYYY HH:mm:ss");
        current_step_container.classList.remove("current");
        current_step_container.classList.add("finished");
        five_summary();
        console.log(response);
      }).catch(reason => {
        e.target.disabled = false;
        document.getElementById("metager-pass-key").disabled = false;
        console.error(reason);
      });
  });
}

function five_summary() {
  let current_step_container = document.getElementById("summary");
  current_step_container.classList.add("current");
}
