const paypal_client = require("@paypal/paypal-js");

function execute_payment_paypal(encrypted_sales_receipts) {
  let payment_method_buttons = document.querySelectorAll(
    "#payment-providers > ul > li"
  );
  for (let i = 0; i < payment_method_buttons.length; i++) {
    payment_method_buttons[i].dataset.active = false;
  }

  let paypal_payment_option_button = document.getElementById(
    "payment_method_paypal"
  );
  paypal_payment_option_button.dataset.active = true;

  let payment_container = document.getElementById("payment-information");
  payment_container.textContent = "";

  let client_id = paypal_payment_option_button.dataset.client_id;
  paypal_client
    .loadScript({
      "client-id": client_id,
      currency: "EUR",
    })
    .then((paypal) => {
      paypal
        .Buttons({
          createOrder: (data, actions) => {
            return fetch("/checkout/payment/order/paypal", {
              method: "POST",
              headers: {
                "Content-Type": "application/json;charset=utf-8",
              },
              body: JSON.stringify({
                order_id: document.querySelector("input[name=order_id]").value,
                expires_at: document.querySelector("input[name=expires_at]")
                  .value,
                amount: document.querySelector("input[name=amount]").value,
                unit_size: document.querySelector("input[name=unit_size]")
                  .value,
                price_per_unit: document.querySelector(
                  "input[name=price_per_unit]"
                ).value,
                public_key_n: document.querySelector("input[name=public_key_n]")
                  .value,
                public_key_e: document.querySelector("input[name=public_key_e]")
                  .value,
                integrity: document.querySelector("input[name=integrity]")
                  .value,
                encrypted_sales_receipts: encrypted_sales_receipts,
              }),
            })
              .then((response) => response.json())
              .then((order) => order.id);
          },
          onCancel: () => cancelPayment(encrypted_sales_receipts),
          onError: () => cancelPayment(encrypted_sales_receipts),
          onApprove: (data, actions) => {
            return fetch("/checkout/payment/order/paypal/capture", {
              method: "POST",
              headers: {
                "Content-Type": "application/json;charset=utf-8",
              },
              body: JSON.stringify({
                order_id: document.querySelector("input[name=order_id]").value,
                amount: document.querySelector("input[name=amount]").value,
                unit_size: document.querySelector("input[name=unit_size]")
                  .value,
                price_per_unit: document.querySelector(
                  "input[name=price_per_unit]"
                ).value,
                public_key_n: document.querySelector("input[name=public_key_n]")
                  .value,
                public_key_e: document.querySelector("input[name=public_key_e]")
                  .value,
                integrity: document.querySelector("input[name=integrity]")
                  .value,
                encrypted_sales_receipts: encrypted_sales_receipts,
              }),
            })
              .then((response) => response.json())
              .then((orderData) => {
                let paymentEvent = new CustomEvent("payment-complete", {
                  detail: {
                    order_id: orderData.order_id,
                    expires_at: data.expires_at,
                    signatures: orderData.signatures,
                  },
                  bubbles: true,
                  cancelable: true,
                  composed: false,
                });
                paypal_payment_option_button.dispatchEvent(paymentEvent);
              });
          },
        })
        .render("#payment-information");
    })
    .catch((err) => {
      // ToDo Handle error
      console.error("failed to load the PayPal JS SDK script", err);
    });
}

function cancelPayment(encrypted_sales_receipts) {
  return fetch("/checkout/payment/order/paypal/cancel", {
    method: "POST",
    headers: {
      "Content-Type": "application/json;charset=utf-8",
    },
    body: JSON.stringify({
      order_id: document.querySelector("input[name=order_id]").value,
      amount: document.querySelector("input[name=amount]").value,
      unit_size: document.querySelector("input[name=unit_size]").value,
      price_per_unit: document.querySelector("input[name=price_per_unit]")
        .value,
      public_key_n: document.querySelector("input[name=public_key_n]").value,
      public_key_e: document.querySelector("input[name=public_key_e]").value,
      integrity: document.querySelector("input[name=integrity]").value,
      encrypted_sales_receipts: encrypted_sales_receipts,
    }),
  });
}

module.exports = execute_payment_paypal;
